/*
 * visrotate.c --
 *
 *	This file implements a rotate visitor. It take the coords
 *	of each item and rotates them around a specified center.
 * 	echo the canvas items in their order. It optionally, takes
 *	a file name.
 *
 *  xp = (x -tx) *cs + (y -ty) *sn + tx
 *  yp = (x -tx) *sn + (y -ty) *cs + ty
 *
 * Known PROBLEMS/flaws:
 *  a) Redefined PTS_IN_ARROW for LineItem
 *  b) Should the -start for the ArcItem be adjusted.
 *  c) ArcItems don't rotate well, in general.
 *  d) RectangleItem, BitmapItem, and ImageItem cannot be rotated.
 *
 * Basically this is most useful for lines, polygons, and symbols.
 *
 * $Header: /nfs/vol/gipsi/src/lib/gptk/RCS/gpRotateVisitor.c,v 1.1 1995/10/04 20:04:59 matt Exp matt $
 */

#include <math.h>
#include "tk.h"
#include "tkVisitor.h"

#ifndef PI
#define PI          (3.1415926535)
#endif /* !PI */
 
#ifndef DEGTORAD
#define DEGTORAD(x) ((x) *PI/180.00000)
#endif /* !DEGTORAD */

typedef struct RotateVisitor {
    Tk_VisitorType *header; /* this field is no longer necessary
			     * when using Tk 4.2 and later */
    double angle;
    double Xcenter;
    double Ycenter;

    double cs;
    double sn;
} RotateVisitor;

#if (TK_MAJOR_VERSION > 4) || (TK_MINOR_VERSION > 1)

/*	NEW DEFINITIONS (Tk 4.2 and later) */
static ClientData RotateStart _ANSI_ARGS_((Tcl_Interp *interp,
		    Tk_Canvas canvas, int argc, char **argv));
static int 	RotateEnd _ANSI_ARGS_((Tcl_Interp *interp,
		    Tk_Canvas canvas, ClientData clientData));
#define		VISITFUNCTION _ANSI_ARGS_((Tcl_Interp *interp, \
		    Tk_Canvas canvas, ClientData clientData, \
		    Tk_Item *itemPtr));
#define		VISITSIZE	sizeof(Tk_VisitorType)
#define		VISITDATA	ClientData
static int 	RotateSticker VISITFUNCTION
static int 	RotateViewport VISITFUNCTION
static int 	RotateBarchart VISITFUNCTION
static int 	RotateStripchart VISITFUNCTION

#else

/*	OLD DEFINITIONS (Tk 4.1) */
static int    	RotateStart _ANSI_ARGS_((Tcl_Interp *interp,
		    Tk_Canvas canvas, Tk_Visitor *clientData,
		    int argc, char **argv));
#define		RotateEnd	(Tk_VisitorEndProc *) NULL
#define		VISITFUNCTION _ANSI_ARGS_((Tcl_Interp *interp, \
		    Tk_Canvas canvas, Tk_Visitor *clientData, \
		    Tk_Item *itemPtr));
#define		VISITSIZE	sizeof(RotateVisitor)
#define		VISITDATA	Tk_Visitor *

#endif

static int 	RotateArc VISITFUNCTION
static int 	RotateBitmap VISITFUNCTION
static int 	RotateImage VISITFUNCTION
static int 	RotateLine VISITFUNCTION
static int 	RotatePolygon VISITFUNCTION
static int 	RotateRectOval VISITFUNCTION
static int 	RotateText VISITFUNCTION
static int 	RotateWindow VISITFUNCTION

Tk_VisitorType tkRotateType = {
	"rotate",			/* name */
	VISITSIZE,			/* visitorSize */
	1,				/* alwaysRedraw */
	RotateStart,			/* startProc */
	RotateEnd,			/* endProc */
#if (TK_MAJOR_VERSION < 5) && (TK_MINOR_VERSION < 4)
	(Tk_VisitorType *) NULL,	/* nextPtr */
#endif
	RotateArc,			/* visitArc */
	RotateBitmap,			/* visitBitmap */
	RotateImage,			/* visitImage */
	RotateLine,			/* visitLine */
	RotateRectOval,			/* visitOval */
	RotatePolygon,			/* visitPolygon */
	RotateRectOval,			/* visitRectangle */
	RotateText,			/* visitText */
	RotateWindow,			/* visitWindow */
#if (TK_MAJOR_VERSION > 4) || (TK_MINOR_VERSION > 1)
	RotateSticker,			/* visitSticker */
	RotateViewport,			/* visitViewport */
	RotateBarchart,			/* visitBarchart */
	RotateStripchart,		/* visitStripchart */
#endif
};


#if defined(__WIN32__)
#   define WIN32_LEAN_AND_MEAN
#   include <windows.h>
#   undef WIN32_LEAN_AND_MEAN

/*
 * VC++ has an alternate entry point called DllMain, so we need to rename
 * our entry point.
 */

#   if defined(_MSC_VER)
#	define EXPORT(a,b) __declspec(dllexport) a b
#	define DllEntryPoint DllMain
#   else
#	if defined(__BORLANDC__)
#	    define EXPORT(a,b) a _export b
#	else
#	    define EXPORT(a,b) a b
#	endif
#   endif
#else
#   define EXPORT(a,b) a b
#endif

EXTERN EXPORT(int,Visrotate_Init) _ANSI_ARGS_((Tcl_Interp *interp));
EXTERN EXPORT(int,Visrotate_SafeInit) _ANSI_ARGS_((Tcl_Interp *interp));

/*
 * These are internal routines.
 */

static void	Transform _ANSI_ARGS_((RotateVisitor *rotatePtr,
		    double *x, double *y));
static void	Transform2 _ANSI_ARGS_((RotateVisitor *rotatePtr,
		    double *bbox));


/*
 *----------------------------------------------------------------------
 *
 * DllEntryPoint --
 *
 *	This wrapper function is used by Windows to invoke the
 *	initialization code for the DLL.  If we are compiling
 *	with Visual C++, this routine will be renamed to DllMain.
 *	routine.
 *
 * Results:
 *	Returns TRUE;
 *
 * Side effects:
 *	None.
 *
 *----------------------------------------------------------------------
 */

#ifdef __WIN32__
BOOL APIENTRY
DllEntryPoint(hInst, reason, reserved)
    HINSTANCE hInst;		/* Library instance handle. */
    DWORD reason;		/* Reason this function is being called. */
    LPVOID reserved;		/* Not used. */
{
    return TRUE;
}
#endif

/*
 *----------------------------------------------------------------------
 *
 * Visrotate_Init --
 *
 *	This procedure is called to add rotation
 *	to the the canvas.
 *
 * Results:
 *	A standard Tcl result.
 *
 * Side effects:
 *	New visitor for rotation is defined.
 *
 *----------------------------------------------------------------------
 */

EXPORT(int,Visrotate_Init)(interp)
    Tcl_Interp *interp;
{
    if (Tcl_PkgRequire(interp, "Tk", TK_VERSION, 1) == NULL) {
	return TCL_ERROR;
    }
    if (Tcl_PkgProvide(interp, "Visrotate", "1.2") != TCL_OK) {
	return TCL_ERROR;
    }
#if (TK_MAJOR_VERSION > 4) || (TK_MINOR_VERSION > 3)
    Tk_CreateCanvasVisitor(interp, (Tk_VisitorType *)&tkRotateType);
#else
    Tk_CreateVisitorType((Tk_VisitorType *)&tkRotateType);
#endif
    return TCL_OK;
}

EXPORT(int,Visrotate_SafeInit)(interp)
    Tcl_Interp *interp;
{
    return Visrotate_Init(interp);
}
/*
 *--------------------------------------------------------------
 *
 * RotateStart --
 *
 *	This procedure is invoked to start a rotate visitor
 *	in a canvas.
 *
 * Results:
 *	A standard Tcl return value.  If an error occurred in
 *	starting the visitor then an error message is left in
 *	canvas->interp->result;
 *
 * Side effects:
 *	none.
 *
 *--------------------------------------------------------------
 */

#if (TK_MAJOR_VERSION > 4) || (TK_MINOR_VERSION > 1)
static ClientData
RotateStart(interp, canvas, argc, argv)
    Tcl_Interp *interp;
    Tk_Canvas canvas;		/* Canvas to hold new item. */
#else
static int
RotateStart(interp, canvas, visitorPtr, argc, argv)
    Tcl_Interp *interp;
    Tk_Canvas canvas;		/* Canvas to hold new item. */
    Tk_Visitor *visitorPtr;	/* Record to hold visitor;  header
				 * must be initialized. */
#endif
    int argc;			/* Number of arguments in argv. */
    char **argv;		/* Arguments for rotate options. */
{
    int length;
#if (TK_MAJOR_VERSION > 4) || (TK_MINOR_VERSION > 1)
    RotateVisitor *rotatePtr = (RotateVisitor *) ckalloc(sizeof(RotateVisitor));

    if (rotatePtr == NULL) {
	Tcl_ResetResult(interp);
	Tcl_AppendResult(interp, "Cannot allocate enough memory for visit rotate command",
			 (char *) NULL);
	return (ClientData) NULL;
    }
#else
    RotateVisitor *rotatePtr = (RotateVisitor *) visitorPtr;
#endif

    rotatePtr->angle = 0.0;
    rotatePtr->Xcenter = 0.0;
    rotatePtr->Ycenter = 0.0;

    while (*argv && !(argc %2)) {
	length = strlen(argv[0]);
	if (strncmp("-angle", argv[0], length) == 0) {
	    if (Tcl_GetDouble(interp, argv[1],
			&(rotatePtr->angle)) != TCL_OK) {
  badValue:
		Tcl_ResetResult(interp);
		Tcl_AppendResult(interp, "bad value \"", argv[1],
				 "\" for option \"", argv[0], "\"",
				 (char *) NULL);
#if (TK_MAJOR_VERSION > 4) || (TK_MINOR_VERSION > 1)
		ckfree((char *) rotatePtr);
		return (ClientData) NULL;
#else
		return TCL_ERROR;
#endif
	    }
	} else if (strncmp("-xcenter", argv[0], length) == 0) {
	    if (Tcl_GetDouble(interp, argv[1],
			&(rotatePtr->Xcenter)) != TCL_OK) {
		goto badValue;
	    }
	} else if (strncmp("-ycenter", argv[0], length) == 0) {
	    if (Tcl_GetDouble(interp, argv[1],
			&(rotatePtr->Ycenter)) != TCL_OK) {
		goto badValue;
	    }
	} else {
  error:
	    Tcl_ResetResult(interp);
	    Tcl_AppendResult(interp, "bad option \"", argv[0],
		"\": must be -angle, -xcenter, or -ycenter",
		(char *) NULL);
#if (TK_MAJOR_VERSION > 4) || (TK_MINOR_VERSION > 1)
	    ckfree((char *) rotatePtr);
	    return (ClientData) NULL;
#else
	    return TCL_ERROR;
#endif
	}
	argc -= 2; argv += 2;
    }
    if (argc != 0) {
	goto error;
    }

    rotatePtr->cs = cos(DEGTORAD(rotatePtr->angle));
    rotatePtr->sn = sin(DEGTORAD(rotatePtr->angle));
#if (TK_MAJOR_VERSION > 4) || (TK_MINOR_VERSION > 1)
    return (ClientData) rotatePtr;
#else
    return TCL_OK;
#endif
}

#if (TK_MAJOR_VERSION > 4) || (TK_MINOR_VERSION > 1)
/*
 *--------------------------------------------------------------
 *
 * RotateEnd --
 *
 *	This procedure is invoked to end a rotate visitor
 *	in a canvas.
 *
 * Results:
 *	A standard Tcl return value.  If an error occurred in
 *	ending the visitor then an error message is left in
 *	canvas->interp->result;
 *
 * Side effects:
 *	none.
 *
 *--------------------------------------------------------------
 */

static int
RotateEnd(interp, canvas, clientData)
    Tcl_Interp *interp;
    Tk_Canvas canvas;
    ClientData clientData;
{
    ckfree((char *) clientData);
    return TCL_OK;
}
#endif

/*
 *--------------------------------------------------------------
 *
 * RotateArc --
 *
 *	This procedure is invoked to rotate an Arc item
 *	in a canvas.
 *
 * Results:
 *	A standard Tcl return value.  If an error occurred in
 *	rotating the item then an error message is left in
 *	canvas->interp->result;  in this case itemPtr is
 *	left unrotated.
 *
 * Side effects:
 *	The coords for the item may change.
 *
 *--------------------------------------------------------------
 */

static int
RotateArc(interp, canvas, clientData, itemPtr)
    Tcl_Interp *interp;
    Tk_Canvas canvas;
    VISITDATA clientData;
    Tk_Item *itemPtr;
{
    RotateVisitor *rotatePtr = (RotateVisitor *) clientData;
    ArcItem *arcPtr = (ArcItem *) itemPtr;

    Transform(rotatePtr, &(arcPtr->bbox[0]), &(arcPtr->bbox[1]));
    Transform(rotatePtr, &(arcPtr->bbox[2]), &(arcPtr->bbox[3]));

    (*itemPtr->typePtr->bboxProc)(canvas, itemPtr);
    return TCL_OK;
}

/*
 *--------------------------------------------------------------
 *
 * RotateBitmap --
 *
 *	This procedure is invoked to rotate an Bitmap item
 *	in a canvas.
 *
 * Results:
 *	A standard Tcl return value.  If an error occurred in
 *	rotating the item then an error message is left in
 *	canvas->interp->result;  in this case itemPtr is
 *	left unrotated.
 *
 * Side effects:
 *	The coords for the item may change.
 *
 *--------------------------------------------------------------
 */

static int
RotateBitmap(interp, canvas, clientData, itemPtr)
    Tcl_Interp *interp;
    Tk_Canvas canvas;
    VISITDATA clientData;
    Tk_Item *itemPtr;
{
    RotateVisitor *rotatePtr = (RotateVisitor *) clientData;
    BitmapItem *bitmapPtr = (BitmapItem *) itemPtr;

    Transform(rotatePtr, &(bitmapPtr->x), &(bitmapPtr->y));

    (*itemPtr->typePtr->bboxProc)(canvas, itemPtr);
    return TCL_OK;
}

/*
 *--------------------------------------------------------------
 *
 * RotateImage --
 *
 *	This procedure is invoked to rotate an Image item
 *	in a canvas.
 *
 * Results:
 *	A standard Tcl return value.  If an error occurred in
 *	rotating the item then an error message is left in
 *	canvas->interp->result;  in this case itemPtr is
 *	left unrotated.
 *
 * Side effects:
 *	The coords for the item may change.
 *
 *--------------------------------------------------------------
 */

static int
RotateImage(interp, canvas, clientData, itemPtr)
    Tcl_Interp *interp;
    Tk_Canvas canvas;
    VISITDATA clientData;
    Tk_Item *itemPtr;
{
    RotateVisitor *rotatePtr = (RotateVisitor *) clientData;
    ImageItem *imagePtr = (ImageItem *) itemPtr;

    Transform(rotatePtr, &(imagePtr->x), &(imagePtr->y));

    (*itemPtr->typePtr->bboxProc)(canvas, itemPtr);
    return TCL_OK;
}

/*
 *--------------------------------------------------------------
 *
 * RotateLine --
 *
 *	This procedure is invoked to rotate an Line item
 *	in a canvas.
 *
 * Results:
 *	A standard Tcl return value.  If an error occurred in
 *	rotating the item then an error message is left in
 *	canvas->interp->result;  in this case itemPtr is
 *	left unrotated.
 *
 * Side effects:
 *	The coords for the item may change.
 *
 *--------------------------------------------------------------
 */

static int
RotateLine(interp, canvas, clientData, itemPtr)
    Tcl_Interp *interp;
    Tk_Canvas canvas;
    VISITDATA clientData;
    Tk_Item *itemPtr;
{
    RotateVisitor *rotatePtr = (RotateVisitor *) clientData;
    LineItem *linePtr = (LineItem *) itemPtr;
    double *coordPtr;
    int i;

    for (i = 0, coordPtr = linePtr->coordPtr; i < linePtr->numPoints;
            i++, coordPtr += 2) {
        Transform(rotatePtr, &(coordPtr[0]), &(coordPtr[1]));
    }
    if (linePtr->firstArrowPtr != NULL) {
        for (i = 0, coordPtr = linePtr->firstArrowPtr; i < PTS_IN_ARROW;
                i++, coordPtr += 2) {
            Transform(rotatePtr, &(coordPtr[0]), &(coordPtr[1]));
        }
    }
    if (linePtr->lastArrowPtr != NULL) {
        for (i = 0, coordPtr = linePtr->lastArrowPtr; i < PTS_IN_ARROW;
                i++, coordPtr += 2) {
            Transform(rotatePtr, &(coordPtr[0]), &(coordPtr[1]));
        }
    }

    (*itemPtr->typePtr->bboxProc)(canvas, itemPtr);
    return TCL_OK;
}

/*
 *--------------------------------------------------------------
 *
 * RotatePolygon --
 *
 *	This procedure is invoked to rotate an Polygon item
 *	in a canvas.
 *
 * Results:
 *	A standard Tcl return value.  If an error occurred in
 *	rotating the item then an error message is left in
 *	canvas->interp->result;  in this case itemPtr is
 *	left unrotated.
 *
 * Side effects:
 *	The coords for the item may change.
 *
 *--------------------------------------------------------------
 */

static int
RotatePolygon(interp, canvas, clientData, itemPtr)
    Tcl_Interp *interp;
    Tk_Canvas canvas;
    VISITDATA clientData;
    Tk_Item *itemPtr;
{
    RotateVisitor *rotatePtr = (RotateVisitor *) clientData;
    PolygonItem *polygonPtr = (PolygonItem *) itemPtr;
    double *coordPtr;
    int i;
 
    for (i = 0, coordPtr = polygonPtr->coordPtr; i < polygonPtr->numPoints;
            i++, coordPtr += 2) {
        Transform(rotatePtr, &(coordPtr[0]), &(coordPtr[1]));
    }

    (*itemPtr->typePtr->bboxProc)(canvas, itemPtr);
    return TCL_OK;
}

/*
 *--------------------------------------------------------------
 *
 * RotateRectOval --
 *
 *	This procedure is invoked to rotate an RectOval item
 *	in a canvas.
 *
 * Results:
 *	A standard Tcl return value.  If an error occurred in
 *	rotating the item then an error message is left in
 *	canvas->interp->result;  in this case itemPtr is
 *	left unrotated.
 *
 * Side effects:
 *	The coords for the item may change.
 *
 *--------------------------------------------------------------
 */

static int
RotateRectOval(interp, canvas, clientData, itemPtr)
    Tcl_Interp *interp;
    Tk_Canvas canvas;
    VISITDATA clientData;
    Tk_Item *itemPtr;
{
    RotateVisitor *rotatePtr = (RotateVisitor *) clientData;
    RectOvalItem *rectovalPtr = (RectOvalItem *) itemPtr;

    Transform2(rotatePtr, &(rectovalPtr->bbox[0]));

    (*itemPtr->typePtr->bboxProc)(canvas, itemPtr);
    return TCL_OK;
}

/*
 *--------------------------------------------------------------
 *
 * RotateText --
 *
 *	This procedure is invoked to rotate an Text item
 *	in a canvas.
 *
 * Results:
 *	A standard Tcl return value.  If an error occurred in
 *	rotating the item then an error message is left in
 *	canvas->interp->result;  in this case itemPtr is
 *	left unrotated.
 *
 * Side effects:
 *	The coords for the item may change.
 *
 *--------------------------------------------------------------
 */

static int
RotateText(interp, canvas, clientData, itemPtr)
    Tcl_Interp *interp;
    Tk_Canvas canvas;
    VISITDATA clientData;
    Tk_Item *itemPtr;
{
    RotateVisitor *rotatePtr = (RotateVisitor *) clientData;
    TextItem *textPtr = (TextItem *) itemPtr;

    Transform(rotatePtr, &(textPtr->x), &(textPtr->y));

    (*itemPtr->typePtr->bboxProc)(canvas, itemPtr);
    return TCL_OK;
}

/*
 *--------------------------------------------------------------
 *
 * RotateWindow --
 *
 *	This procedure is invoked to rotate an Window item
 *	in a canvas.
 *
 * Results:
 *	A standard Tcl return value.  If an error occurred in
 *	rotating the item then an error message is left in
 *	canvas->interp->result;  in this case itemPtr is
 *	left unrotated.
 *
 * Side effects:
 *	The coords for the item may change.
 *
 *--------------------------------------------------------------
 */

static int
RotateWindow(interp, canvas, clientData, itemPtr)
    Tcl_Interp *interp;
    Tk_Canvas canvas;
    VISITDATA clientData;
    Tk_Item *itemPtr;
{
    RotateVisitor *rotatePtr = (RotateVisitor *) clientData;
    WindowItem *windowPtr = (WindowItem *) itemPtr;

    Transform(rotatePtr, &(windowPtr->x), &(windowPtr->y));

    (*itemPtr->typePtr->bboxProc)(canvas, itemPtr);
    return TCL_OK;
}

#if (TK_MAJOR_VERSION > 4) || (TK_MINOR_VERSION > 1)
/*
 *--------------------------------------------------------------
 *
 * RotateSticker --
 *
 *	This procedure is invoked to rotate an Sticker item
 *	in a canvas.
 *
 * Results:
 *	A standard Tcl return value.  If an error occurred in
 *	rotating the item then an error message is left in
 *	canvas->interp->result;  in this case itemPtr is
 *	left unrotated.
 *
 * Side effects:
 *	The coords for the item may change.
 *
 *--------------------------------------------------------------
 */

static int
RotateSticker(interp, canvas, clientData, itemPtr)
    Tcl_Interp *interp;
    Tk_Canvas canvas;
    ClientData clientData;
    Tk_Item *itemPtr;
{
    RotateVisitor *rotatePtr = (RotateVisitor *) clientData;
    StickerItem *stickerPtr = (StickerItem *) itemPtr;

    Transform2(rotatePtr, &(stickerPtr->bbox[0]));

    (*itemPtr->typePtr->bboxProc)(canvas, itemPtr);
    return TCL_OK;
}

/*
 *--------------------------------------------------------------
 *
 * RotateViewport --
 *
 *	This procedure is invoked to rotate an Viewport item
 *	in a canvas.
 *
 * Results:
 *	A standard Tcl return value.  If an error occurred in
 *	rotating the item then an error message is left in
 *	canvas->interp->result;  in this case itemPtr is
 *	left unrotated.
 *
 * Side effects:
 *	The coords for the item may change.
 *
 *--------------------------------------------------------------
 */

static int
RotateViewport(interp, canvas, clientData, itemPtr)
    Tcl_Interp *interp;
    Tk_Canvas canvas;
    ClientData clientData;
    Tk_Item *itemPtr;
{
    RotateVisitor *rotatePtr = (RotateVisitor *) clientData;
    PortItem *viewportPtr = (PortItem *) itemPtr;

    Transform2(rotatePtr, &(viewportPtr->bbox[0]));

    (*itemPtr->typePtr->bboxProc)(canvas, itemPtr);
    return TCL_OK;
}

/*
 *--------------------------------------------------------------
 *
 * RotateBarchart --
 *
 *	This procedure is invoked to rotate an Barchart item
 *	in a canvas.
 *
 * Results:
 *	A standard Tcl return value.  If an error occurred in
 *	rotating the item then an error message is left in
 *	canvas->interp->result;  in this case itemPtr is
 *	left unrotated.
 *
 * Side effects:
 *	The coords for the item may change.
 *
 *--------------------------------------------------------------
 */

static int
RotateBarchart(interp, canvas, clientData, itemPtr)
    Tcl_Interp *interp;
    Tk_Canvas canvas;
    ClientData clientData;
    Tk_Item *itemPtr;
{
    RotateVisitor *rotatePtr = (RotateVisitor *) clientData;
    BarchartItem *barchartPtr = (BarchartItem *) itemPtr;

    Transform2(rotatePtr, &(barchartPtr->bbox[0]));

    (*itemPtr->typePtr->bboxProc)(canvas, itemPtr);
    return TCL_OK;
}

/*
 *--------------------------------------------------------------
 *
 * RotateStripchart --
 *
 *	This procedure is invoked to rotate an Stripchart item
 *	in a canvas.
 *
 * Results:
 *	A standard Tcl return value.  If an error occurred in
 *	rotating the item then an error message is left in
 *	canvas->interp->result;  in this case itemPtr is
 *	left unrotated.
 *
 * Side effects:
 *	The coords for the item may change.
 *
 *--------------------------------------------------------------
 */

static int
RotateStripchart(interp, canvas, clientData, itemPtr)
    Tcl_Interp *interp;
    Tk_Canvas canvas;
    ClientData clientData;
    Tk_Item *itemPtr;
{
    RotateVisitor *rotatePtr = (RotateVisitor *) clientData;
    StripchartItem *stripchartPtr = (StripchartItem *) itemPtr;

    Transform2(rotatePtr, &(stripchartPtr->bbox[0]));

    (*itemPtr->typePtr->bboxProc)(canvas, itemPtr);
    return TCL_OK;
}
#endif

/*
 *--------------------------------------------------------------
 *
 * Transform --
 *
 *	This procedure is called to perform the transformation to the
 *	x/y pair.
 *
 * Results:
 *	None.
 *
 * Side effects:
 *	None.
 *
 *--------------------------------------------------------------
 */

static void
Transform(rotatePtr, x, y)
    RotateVisitor *rotatePtr;
    double *x, *y;
{
    double tmp;

    *x = *x -rotatePtr->Xcenter;
    *y = *y -rotatePtr->Ycenter;

    tmp = *x;
    *x =  (*x  *rotatePtr->cs) +(*y *rotatePtr->sn) +rotatePtr->Xcenter;
    *y = -(tmp *rotatePtr->sn) +(*y *rotatePtr->cs) +rotatePtr->Ycenter;
}

/*
 *--------------------------------------------------------------
 *
 * Transform2 --
 *
 *	This procedure is called to perform the transformation of
 *	a bounding box, keeping the orientation equal.
 *
 * Results:
 *	None.
 *
 * Side effects:
 *	None.
 *
 *--------------------------------------------------------------
 */

static void
Transform2(rotatePtr, bbox)
    RotateVisitor *rotatePtr;
    double *bbox;
{
    double x,y,tmp;

    x = (bbox[2]+bbox[0])/2 - rotatePtr->Xcenter;
    y = (bbox[3]+bbox[1])/2 - rotatePtr->Ycenter;

    tmp = x;
    x -=  (x  * rotatePtr->cs) + (y * rotatePtr->sn);
    y -=  -(tmp * rotatePtr->sn) + (y * rotatePtr->cs);

    bbox[0] -= x;
    bbox[1] -= y;
    bbox[2] -= x;
    bbox[3] -= y;
}
